/*
  ------------------------------------------------------------------------------

    Copyright © 2005-2012, Tommi T. Järvi
                           firstname . jarvi @ iki . fi (remove spaces)


    This file is part of one of my programs.

    This is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2,
    as published by the Free Software Foundation.

    This is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this file; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

  ------------------------------------------------------------------------------
*/


#include <valarray>
#include <iostream>
#include <string>
#include <vector>
#include "mdconstants.h"
#include "global.h"
#include "mathsubs.h"


using namespace std;


#ifndef DEF__ATOM_TYPES
#define DEF__ATOM_TYPES

class Atom_Types {

  /*
    Defines the atom types in a simulation.
  */

 public:
  int maxtypes;  // highest type = maxtypes-1

  int ntypes;  // number of atom types
  int *types;  // types[0,ntypes-1], atom type numbers visible outside
               //   values in [0,highest-type] since act as indices
               //   for arrays below

  // arrays of size maxtypes with data in [0,highest_type]
  // (atom types not in a particular order)
  vector<string> atypename;  // names of atom types
  double *mass;  // masses
  int *Z;  // atomic numbers
  int *natype;  // number of atoms of a type

  // --- operators

  Atom_Types();
  ~Atom_Types();
  Atom_Types(const Atom_Types &syst);
  Atom_Types& operator=(const Atom_Types &syst);

  // --- functions

  void add(string name, int &n);  // only use this for adding atom types
  void reset();  // remove atom types and reset everything
  void zero();  // set natype[i] = 0 but keep atom types
  void setmasses(bool warn = false);  // set masses, for warn = true, no error if not found, just set to zero
  void setmass(int i, double m);  // set mass of type i

  // --- queries
  int typenumber(string name);  // Case sensitive!
  string name(int n);  // atom type name of type n
  void printtypes();  // output atom types and number of atoms of each type
  int freenumber();  // give a free type number
  bool safeadd(string name, int n);  // is it safe to add this type?
  int highest_type() {return maxtypes-1;}  // get highest possible type number (range [0,given number])
};
#endif
